require 'helper'

module Monark
  class ContextTest < MiniTest::Unit::TestCase

    include TestHelp

    def teardown
      # TODO: Add as method to Schema class
      ActiveRecord::Base.connection.execute "DROP TABLE IF EXISTS monark_info"
      Thread.current[:counter] = nil
    end

    test 'should perform migrations' do
      Thread.current[:table] = 'foo'
      transaction do
        context = Context.new(fixtures_path)
        context.migrate
        schema = Schema.new
        assert_equal 3, schema.next_migration
        assert_equal 1, schema.version
      end
    end

    test '#done should set the next migration past the current ones' do
      transaction do
        context = Context.new(fixtures_path)
        context.done
        assert_equal 3, Schema.new.next_migration
      end
    end

    test '#migrate should skip completed migrations' do
      transaction do
        schema = Schema.new
        schema.next_migration = 1
        context = Context.new(fixtures_path)
        Thread.current[:counter] = 0
        context.manifest = <<-END
          add 'foo' do
            Thread.current[:counter] = Thread.current[:counter].next
          end
          add 'bar' do
            Thread.current[:counter] = Thread.current[:counter].next
          end
          add 'baz' do
            Thread.current[:counter] = Thread.current[:counter].next
          end
        END
        context.migrate
        assert_equal 2, Thread.current[:counter]
      end
    end

    test '#migrate should optionally reload migrations added with load' do
      transaction do
        schema = Schema.new
        schema.next_migration = 100
        context = Context.new(fixtures_path)
        Thread.current[:counter] = 0
        context.manifest = <<-END
          load 'foo' do
            Thread.current[:counter] = Thread.current[:counter].next
          end
        END
        context.migrate
        assert_equal 0, Thread.current[:counter]
        assert_equal 100, schema.next_migration
        context.migrate(:reload => true)
        assert_equal 100, schema.next_migration
        assert_equal 1, Thread.current[:counter]
      end
    end

    test 'should have a default migrations file name' do
      assert_equal 'Migrations.rb', DEFAULT_MIGRATIONS_FILE_NAME
    end

    test 'constructor should set a root Pathname' do
      assert_equal Pathname, context.root.class
      assert_equal '/tmp', context.root.to_s
    end

    test 'constructor should set a migrations file name' do
      context = Context.new('/tmp', 'foo.rb')
      assert_equal 'foo.rb', context.migrations_file_name
    end

    test 'constructor should set default migrations file name' do
      assert_equal DEFAULT_MIGRATIONS_FILE_NAME, context.migrations_file_name
    end

    test 'should load the migrations file as the manifest' do
      context = Context.new(fixtures_path)
      data = File.read(fixtures_path.join('Migrations.rb'))
      assert_equal data, context.manifest
    end

    test 'should push to the migrations array' do
      context.push :foo
      assert_equal :foo, context.migrations.first
    end

    test 'should set version from number or numeric string' do
      context.version = 1
      assert_equal 1, context.version
      context.version = '2'
      assert_equal 2, context.version
    end

    test 'should raise error if the version can not be cast to a number' do
      begin
        context.version = 'foo'
        flunk 'should have raised an error'
      rescue ArgumentError
        assert true
      end
    end

    test 'should load manifest' do
      context = Context.new(fixtures_path)
      context.load_manifest
      assert_equal 42, context.version
      assert_equal 3, context.migrations.count
    end
  end
end